/*
 * Project name: Stand-alone pulse meter using Easy Pulse mikro
       -  ported for clicker 2 for PIC18FJ(FJ);
 * Description:  Find details at:
  http://embedded-lab.com/blog/easy-pulse-mikro/

 * Test configuration:
     MCU:             PIC18F87J50
                      http://ww1.microchip.com/downloads/en/DeviceDoc/39775b.pdf
     Dev. Board       clicker 2 for PIC18FJ
                      http://www.mikroe.com/pic/clicker-2-pic18fj
     Oscillator:      HS-PLL 48.0000 MHz, 8.0000 MHz Crystal
     ext. modules:    OLED W click, Easy Pulse mikro

     SW:              mikroC PRO for PIC

 NOTES:
     - Place OLED W Click board in the mikroBUS socket 1.
     - Place Easy Pulse mikro in the mikroBUS socket 2.
*/

#include "OLED.h"

sbit OLED_RST at LATD2_bit;
sbit OLED_CS at LATD3_bit;
sbit OLED_DC at LATG3_bit;
sbit OLED_RST_Direction at TRISD2_bit;
sbit OLED_CS_Direction at TRISD3_bit;
sbit OLED_DC_Direction at TRISG3_bit;

int ADC_Samples[Num_Samples], Index1, Index2, Index3, Peak1, Peak2, Peak3;
long PPlong, Temp1=1L, Pulse_Time1, Pulse_Time2;
int Peak_Magnitude, Peak_Threshold, Minima, Range, iloop, m,Pulse_Rate;

// Read ADC samples at a predefined interval
 void Read_ADC_Samples(){
   for (iloop = 0; iloop < Num_Samples; iloop++){
      ADC_Samples[iloop] = ADC_Read(1);
      Delay_ms(Sampling_Time);
   }
 }

void Find_Minima(int Num){
  Minima = 1024;
  for (m = Num; m < Num_Samples-Num; m++){
      if(Minima > ADC_Samples[m]){
         Minima = ADC_Samples[m];
      }
  }
}  // Find_Minima

 void Remove_DC(){
   Find_Minima(0);

   // Subtract DC (minima)
   for (iloop = 0; iloop < Num_Samples; iloop++){
     ADC_Samples[iloop] = ADC_Samples[iloop] - Minima;
   }
   Minima = 0;  // New minima is zero
 }  // Remove_DC

 void Find_Peak(int Num){
  Peak_Magnitude = 0;
  for (m = Num; m < Num_Samples-Num; m++){
      if(Peak_Magnitude < ADC_Samples[m]){
        Peak_Magnitude = ADC_Samples[m];
     }
  }
} // Find_Peak

void Scale_Data(){
  // Find peak value
  Find_Peak(0);
  Range = Peak_Magnitude - Minima;
  // Sclae from 1 to 1023
  for (iloop = 0; iloop < Num_Samples; iloop++){
   //  ADC_Samples[iloop] = map(ADC_Samples[iloop], 0, Range, 1, 1023);


  }
  Find_Peak(0);
  Find_Minima(0);

} // Scale_Data



void Filter_Data(){
  int Num_Points = 2*Moving_Average_Num+1;
  int Sum_Points, k;
  for (iloop = Moving_Average_Num; iloop < Num_Samples-Moving_Average_Num; iloop++){
    Sum_Points = 0;
    for(k =0; k < Num_Points; k++){
      Sum_Points = Sum_Points + ADC_Samples[iloop-Moving_Average_Num+k];
    }
    ADC_Samples[iloop] = Sum_Points/Num_Points;
  }
}  // Filter_Date


void Compute_Pulse_Rate(){
  // Detect Peak magnitude and minima
  Find_Peak(Moving_Average_Num);
  Find_Minima(Moving_Average_Num);
  Range = Peak_Magnitude - Minima;
  Peak_Threshold = Peak_Magnitude*Peak_Threshold_Factor;
  Peak_Threshold = Peak_Threshold/100;

  // Now detect three peaks
  Peak1 = 0;
  Peak2 = 0;
  Peak3 = 0;
  Index1 = 0;
  Index2 = 0;
  Index3 = 0;
  // Find first peak
  for (m = Moving_Average_Num; m < Num_Samples-Moving_Average_Num; m++){
      if(ADC_Samples[m] >= ADC_Samples[m-1] && ADC_Samples[m] > ADC_Samples[m+1] &&
         ADC_Samples[m] > Peak_Threshold && Peak1 == 0){
        Peak1 = ADC_Samples[m];
        Index1 = m;
      }

      // Search for second peak which is at least 10 sample time far
      if(Peak1 > 0 && m > (Index1+Minimum_Peak_Separation) && Peak2 == 0){
         if(ADC_Samples[m] >= ADC_Samples[m-1] && ADC_Samples[m] > ADC_Samples[m+1] &&
         ADC_Samples[m] > Peak_Threshold){
         Peak2 = ADC_Samples[m];
         Index2 = m;
         }
      } // Peak1 > 0

      // Search for the third peak which is at least 10 sample time far
      if(Peak2 > 0 && m > (Index2+Minimum_Peak_Separation) && Peak3 == 0){
         if(ADC_Samples[m] >= ADC_Samples[m-1] && ADC_Samples[m] > ADC_Samples[m+1] &&
         ADC_Samples[m] > Peak_Threshold){
            Peak3 = ADC_Samples[m];
            Index3 = m;
         }
      } // Peak2 > 0

  }



  Pulse_Time1 = (Index2-Index1)*Sampling_Time; // In milliseconds
  Pulse_Time2 = (Index3-Index2)*Sampling_Time;

  if(Pulse_Time1 > 0 && Pulse_Time1 > 0){
    PPlong = 2*60000/(Pulse_Time1+Pulse_Time2); // In BPM
    Pulse_Rate=PPlong;
  }

}  // Compute_Pulse_Rate






//Send command to OLED m display
void OLED_M_command(unsigned char temp){
  OLED_CS=0;
  OLED_DC=0;
  SPI1_Write(temp);
  OLED_CS=1;
}
//Send data to OLED m display
void OLED_M_data(unsigned char temp){
  OLED_CS=0;
  OLED_DC=1;
  SPI1_Write(temp);
  OLED_CS=1;
}

void OLED_M_Init() {
  OLED_RST=0;
  Delay_ms(100);
  OLED_RST=1;
  Delay_ms(100);
  OLED_M_command(SSD1306_DISPLAYOFF);             //0xAE  Set OLED Display Off
  OLED_M_command(SSD1306_SETDISPLAYCLOCKDIV);     //0xD5  Set Display Clock Divide Ratio/Oscillator Frequency
  OLED_M_command(0x80);
  OLED_M_command(SSD1306_SETMULTIPLEX);           //0xA8  Set Multiplex Ratio
  OLED_M_command(39);
  OLED_M_Command(SSD1306_SETSEGMENTREMAP);        //0xA1  Set Segment Remap Inv
  OLED_M_Command(SSD1306_COMSCANDEC);             //0xC8  Set COM Output Scan Inv
  OLED_M_Command(SSD1306_SETSTARTLINE);           //0x40  Set Display Start Line
  OLED_M_command(SSD1306_SETDISPLAYOFFSET);       //0xD3  Set Display Offset
  OLED_M_command(0x00);
  //OLED_M_command(SSD1306_SETSTARTLINE);           //0x40  Set Display Start Line
  OLED_M_command(SSD1306_CHARGEPUMP);             //0x8D  Set Charge Pump
  OLED_M_command(0x14);                           //0x14  Enable Charge Pump
  //OLED_M_command(SSD1306_COMSCANDEC);             //0xC8  Set COM Output Scan Direction
  OLED_M_command(SSD1306_SETCOMPINS);             //0xDA  Set COM Pins Hardware Configuration
  OLED_M_command(0x12);
  OLED_M_command(SSD1306_SETCONTRAST);            //0x81   Set Contrast Control
  OLED_M_command(0xAF);
  OLED_M_command(SSD1306_SETPRECHARGE);           //0xD9   Set Pre-Charge Period
  OLED_M_command(0x25);
  OLED_M_command(SSD1306_SETVCOMDETECT);          //0xDB   Set VCOMH Deselect Level
  OLED_M_command(0x20);
  OLED_M_command(SSD1306_DISPLAYALLON_RESUME);    //0xA4   Set Entire Display On/Off
  OLED_M_command(SSD1306_NORMALDISPLAY);          //0xA6   Set Normal/Inverse Display
  OLED_M_command(SSD1306_DISPLAYON);              //0xAF   Set OLED Display On
}


//Set page adress for Page Addressing Mode
void Set_Page_Address(unsigned char add) {
  add=0xb0|add;
  OLED_M_command(add);
}
//Set column adress for Page Addressing Mode
void Set_Column_Address(unsigned char add) {
  add += 32;
  OLED_M_command((0x10|(add>>4)));
  OLED_M_command((0x0f&add));
}


//Display picture for Page Addressing Mode
void Display_Picture(unsigned char pic[]) {
  unsigned char i,j;
  for(i=0;i<0x05;i++) {
    Set_Page_Address(i);
    // Set_Column_Address(0x00);
    OLED_M_command(0x10);
    OLED_M_command(0x40);
    for(j=0;j<0x60;j++) {
      OLED_M_data(pic[i*0x60+j]);
    }
  }
}
//Set contrast control
void contrast_control(unsigned char temp){
  OLED_M_command(SSD1306_SETCONTRAST);           //0x81   Set Contrast Control
  OLED_M_command(temp);                          // contrast step 1 to 256
}

void startscroll_Right(unsigned char x, unsigned char y){
  OLED_M_command(SSD1306_RIGHT_HORIZONTAL_SCROLL);      //0x26  Right Horizontal scroll
  OLED_M_command(0X00);                                 //dummy byte
  OLED_M_command(x);                                    //define start page address
  OLED_M_command(0X00);                                 //Set time interval between each scroll
  OLED_M_command(y);                                    //Define end page address
  OLED_M_command(0X00);                                 //dummy byte
  OLED_M_command(0XFF);                                 //dummy byte
  OLED_M_command(SSD1306_ACTIVATE_SCROLL);
}

void startscroll_Left(unsigned char x, unsigned char y){
  OLED_M_command(SSD1306_LEFT_HORIZONTAL_SCROLL);       //0x27  Right Horizontal scroll
  OLED_M_command(0X00);                                 //dummy byte
  OLED_M_command(x);                                    //define start page address
  OLED_M_command(0X00);                                 //Set time interval between each scroll
  OLED_M_command(y);                                    //Define end page address
  OLED_M_command(0X00);                                 //dummy byte
  OLED_M_command(0XFF);                                 //dummy byte
  OLED_M_command(SSD1306_ACTIVATE_SCROLL);              //0x2F Activate scroll
}

void startscroll_DiagRight(unsigned char x, unsigned char y){
  OLED_M_command(SSD1306_SET_VERTICAL_SCROLL_AREA);              //0xA3 Set Vertical Scroll Area
  OLED_M_command(0X00);                                          //Set No. of rows in top fixed area
  OLED_M_command(SSD1306_LCDHEIGHT);                             //Set No. of rows in scroll area
  OLED_M_command(SSD1306_VERTICAL_AND_RIGHT_HORIZONTAL_SCROLL);  //0x29 Vertical and Right Horizontal Scroll
  OLED_M_command(0X00);                                          //dummy byte
  OLED_M_command(x);                                             //Define start page address
  OLED_M_command(0X00);                                          //Set time interval between each scroll
  OLED_M_command(y);                                             //Define end page address
  OLED_M_command(0X01);                                          //Vertical scrolling offset
  OLED_M_command(SSD1306_ACTIVATE_SCROLL);                       //0x2F Activate scroll
}

void startscroll_DiagLeft(unsigned char x, unsigned char y){
  OLED_M_command(SSD1306_SET_VERTICAL_SCROLL_AREA);              //0xA3 Set Vertical Scroll Area
  OLED_M_command(0X00);                                          //Set No. of rows in top fixed area
  OLED_M_command(SSD1306_LCDHEIGHT);                             //Set No. of rows in scroll area
  OLED_M_command(SSD1306_VERTICAL_AND_LEFT_HORIZONTAL_SCROLL);   //0x2A Vertical and Right Horizontal Scroll
  OLED_M_command(0X00);                                          //dummy byte
  OLED_M_command(x);                                             //Define start page address
  OLED_M_command(0X00);                                          //Set time interval between each scroll
  OLED_M_command(y);                                             //Define end page address
  OLED_M_command(0X01);                                          //Vertical scrolling offset
  OLED_M_command(SSD1306_ACTIVATE_SCROLL);                       //2F Activate scroll
}

void scroll_STOP(){
  OLED_M_command(0x2E);    //0x2E deactivate scroll
}

char x, y;

void OLED_Clear( void)
{
    unsigned char i,j;
    for( i=0; i<5; i++) // 5*8=40 pixel rows (actually 39)
    {
        Set_Page_Address( i);
        Set_Column_Address( 0);
        for( j=0; j<96; j++)  OLED_M_Data( 0);
    }
    x = 0; y = 0;
    Set_Page_Address(0);
    Set_Column_Address(0);
}

void OLED_Putchar( char ch)
{
    unsigned short i;
    const char *f = &font[(ch-' ')*5];
    for( i=0; i<5; i++)
        OLED_M_Data( *f++ << 1);
    OLED_M_Data( 0);
    x++;
    if (x >= 32) { // wrap x
        x = 0; Set_Column_Address(0);
        y++;
        if (y >= 5) { // wrap y
            y = 0;
        }
        Set_Page_Address(y);
    }
}

void OLED_Puts( char x1, char y1, char *s)
{
    y = y1; x = x1;
    Set_Page_Address(y);
    Set_Column_Address(x*6);
    while( *s) {
        OLED_Putchar( *s++);
        x++;
    }
}

void InitMCU(){     //  Init MCU function
  OLED_RST_Direction = 0;
  OLED_CS_Direction = 0;
  OLED_DC_Direction = 0;

  SPI1_Init_Advanced(_SPI_MASTER_OSC_DIV64, _SPI_DATA_SAMPLE_MIDDLE, _SPI_CLK_IDLE_LOW, _SPI_LOW_2_HIGH);
  // Set ADC input
  TRISA.F1 = 1;   // RA1 is AN1 input
  ANCON0 = 0b11111110;
  ADC_Init();
  ADCON0 = 0b00000100;
  Delay_ms(500);
}

void main(){
  char txt[7];
  PLLEN_bit=1;
  Delay_ms(150);
  InitMCU();
  OLED_M_Init();
  OLED_Clear();
  OLED_Puts( 0, 0, "EASY PULSE MIKRO");
  while(1){
    
    Read_ADC_Samples();
    Remove_DC();
    Scale_Data();

  if (Range > Minimum_Range){  // ADC range is > 70, otherwise increase gain
       Filter_Data();
       Compute_Pulse_Rate();
       IntToStr(Pulse_Rate, txt);
       OLED_Clear();
       OLED_Puts( 0, 0, "EASY PULSE MIKRO");
       OLED_Puts( 0, 2, txt);
       OLED_Puts( 9, 2, "BPM");
  }
  else{
    OLED_Clear();
    OLED_Puts( 0, 0, "EASY PULSE MIKRO");
    OLED_Puts( 0, 2, "ERROR!");
    OLED_Puts( 0, 3, "CHECK SENSOR");
    OLED_Puts( 0, 4, "CONNECTION ...");
  }
    
    Delay_ms(500);
  }
}